#!/usr/bin/ruby
# encoding: utf-8

require 'glib2'
require 'gtk2'
require 'vte'
require 'MfsDiskDrive' 
require 'MfsSinglePartition'

def info_dialog(title, text) 
	dialog = Gtk::Dialog.new(
		title,
		$mainwindow,
		Gtk::Dialog::MODAL,
		[ Gtk::Stock::OK, Gtk::Dialog::RESPONSE_OK ]
	)
	dialog.default_response = Gtk::Dialog::RESPONSE_OK
	dialog.has_separator = false
	label = Gtk::Label.new
	label.set_markup(text)
	label.wrap = true
	image = Gtk::Image.new(Gtk::Stock::DIALOG_INFO, Gtk::IconSize::DIALOG)
	hbox = Gtk::HBox.new(false, 5)
	hbox.border_width = 10
	hbox.pack_start_defaults(image)
	hbox.pack_start_defaults(label)
	dialog.vbox.add(hbox)
	dialog.show_all
	dialog.run { |resp|
		dialog.destroy
		return true
	}
end

def read_temp
	current = nil
	critical = nil
	# CPU Temperature:    +59.0°C  (high = +60.0°C, crit = +95.0°C)
	IO.popen("sensors") { |l|
		while l.gets 
			if $_ =~  /.*?\:.*?\+(\d*?\.\d)°C.*?crit.*?\+(\d*?\.\d)°C/i
				current = $1.to_f
				critical = $2.to_f
			end
		end
	}
	# puts current.to_s
	# puts critical.to_s 
	return current, critical 
end


def run_smart_test(pgbar, templabel)
	tremain = 0
	@drives.each { |d|
		t = d.smart_short_test 
		unless t.nil?
			tremain = t if t > tremain
		end
	}
	exit 1 if tremain < 1
	current, critical = read_temp
	exit 1 if current.nil?
	vte = Vte::Terminal.new
	vte.fork_command("cpuburn") 
	tremain += 1 
	tstart = Time.now
	cycles = 0 
	while @running == true
		if (cycles % 5 == 0)
			current, critical = read_temp
			templabel.set_markup("Aktuelle CPU-Temperatur: <b>#{current}°C</b> (kritische Grenze: #{critical}°C)") 
			if @cpuburn == true && (critical - current < 5.0)
				system("killall -9 cpuburn")
				info_dialog("Temperatur zu hoch!", "Die CPU-Temperatur ist zu hoch, der Lasttest wurde abgebrochen. Bitte überprüfen Sie die Kühlung!") 	
				@cpuburn = false
			end
			tnow = Time.now
			pgbar.fraction = (tnow - tstart) / (tremain.to_f + 1.0) / 60.0
			exit 0 if ( (tnow - tstart) / (tremain.to_f + 1.0) / 60.0 > 1.0 )
			while (Gtk.events_pending?)
				Gtk.main_iteration
			end
			sleep 0.2
		end
		cycles += 1
	end
end

@drives = Array.new
Dir.entries("/sys/block").each { |l|
	@drives.push(MfsDiskDrive.new(l, true)) if l =~ /[a-z]$/ 
}
Dir.entries("/sys/block").each { |l|
	@drives.push(MfsDiskDrive.new(l, true)) if ( l =~ /mmcblk[0-9]$/ ||  l =~ /mmcblk[0-9][0-9]$/ )
}
	
@running = true 
@cpuburn = true 

# Description

winvbox = Gtk::VBox.new(false, 3)
desc = Gtk::Label.new
desc.wrap = true
desc.width_request = 480
desc.set_markup("Im Hintergrund wurde ein <b>Schnelltest Ihrer Festplatte(n)</b> gestartet. Ein <b>Lasttest der CPU</b> dient dazu, Probleme mit der Kühlung aufzuspüren. Sie können beide Tests abbrechen. Der Festplattentest läuft dann im Hintergrund weiter (Ergebnisse lesen Sie mit dem Werkzeug \"Rettungswerkzeuge -> Platte testen\" aus).")
winvbox.pack_start_defaults(desc)

# Temperature

templabel = Gtk::Label.new 
winvbox.pack_start_defaults(templabel)

# Progressbar

pgbar = Gtk::ProgressBar.new
pgbar.text = "Schnelltest der Festplatte(n)"
cancel = Gtk::Button.new(Gtk::Stock::CANCEL)
pbox = Gtk::HBox.new(false, 3)
pbox.pack_start(pgbar, true, true, 3)
pbox.pack_start(cancel, false, true, 3)
winvbox.pack_start_defaults(pbox)

window = Gtk::Window.new
window.signal_connect("delete_event") {
        puts "delete event occurred"
        false
}

window.signal_connect("destroy") {
        puts "destroy event occurred"
        Gtk.main_quit
}

cancel.signal_connect("clicked") { exit 0 }

window.border_width = 10
# window.width_request = 500 
window.set_title("Stresstest")
window.window_position = Gtk::Window::POS_CENTER_ALWAYS
window.add(winvbox)
GLib::Timeout.add(100){ run_smart_test(pgbar, templabel) }
window.show_all
Gtk.main


